#include <stdio.h>
#include "jcam_dll.h"

using namespace joescan;

int main(int argc, char **argv)
{
#ifdef JCAM_STATIC_LIB
	if(jsInitialize() == FALSE)
    {
        printf("Could not initialize TCP\\IP communications.\n");
        return -1;
    }
#endif
    char *ip = "192.168.1.150";
    if(argc > 1)
        ip = argv[1];

    JCONNECTION jc = jsOpenConnection(ip);
    if(jc == 0)
    {
        printf("Could not open connection to %s.\n", ip);
        return -1;
    }

    switch(jsSendParameterFileToScanner(jc, "param.dat"))
    {
        case SCANNER_FAILURE:
            printf("Lost connection.\n");
            jsCloseConnection(jc);
            jc = 0;
            return -1;
        case OPERATION_FAILURE:
            for(int i = 0; i < jsGetNumberOfErrorMessages(jc); i++)
                printf("%s\n", jsGetErrorMessage(jc, i));
            jsCloseConnection(jc);
            jc = 0;
            return -1;
        default:
            printf("Sent parameters successfully.\n");
    }

    switch(jsEnterEncoderSyncMode(jc))
    {
        case 0:
            break;
        case INVALID_PARAMETER:
            printf("Invalid parameter to jsEnterEncoderSyncMode().\n");
            jsCloseConnection(jc);
            jc = 0;
            return -1;
        case SCANNER_FAILURE:
            printf("Scanner failure during jsEnterEncoderSyncMode().\n");
            jsCloseConnection(jc);
            jc = 0;
            return -1;
    }

    jsProfile profile;
    while (true) // it would be better to run this loop while some external condition is true, 
		// e.g. while a PLC tells us that an object is in the scan zone, otherwise we will never exit
		// out of the loop. Another approach is to look at the contents of profiles and decide 
		// if the object of interest is still in the scan zone. Breaking the loop when 
		// a certain number of empty profiles have been received is a good way to do this. 
    {
        switch(jsGetProfile(jc, &profile))
        {
            case 0:
                //logModel.addProfile(profile);
                // no error occured and we got one profile, this is where you can 
				// add the profile to your model of the object being scanned
				break;
            case PROFILE_UNAVAILABLE:
                //no new profiles have been measured yet, but that is not an error. 
				//We could use this time to look at our model.
				//optimizer.earlyOptimize(logModel);				
                break;
            case INVALID_PARAMETER:
                printf("Invalid parameter to jsGetProfile().\n");
                jsCloseConnection(jc);
                jc = 0;
                return -1;
            case SCANNER_FAILURE:
                printf("Scanner failure during jsGetProfile().\n");
                jsCloseConnection(jc);
                jc = 0;
                return -1;
        }
    }

    switch(jsExitSyncMode(jc))
    {
        case 0:
            break;
        case INVALID_PARAMETER:
            printf("Invalid parameter to jsExitSyncMode().\n");
            jsCloseConnection(jc);
            jc = 0;
            return -1;
        case SCANNER_FAILURE:
            printf("Scanner failure during jsExitSyncMode().\n");
            jsCloseConnection(jc);
            jc = 0;
            return -1;
    }

    jsCloseConnection(jc);
    jc = 0;
    //we left sync mode, and have all data in a model, we're now ready to process that model
	//optimizer.fullyOptimize(logModel);
    
	return 0;
}